package saccubus.worker;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.concurrent.Callable;
import java.util.logging.Logger;
import javax.swing.SwingWorker;
import nicobrowser.GetFlvResult;
import nicobrowser.NamePattern;
import nicobrowser.NicoHttpClient;
import nicobrowser.ProgressListener;
import nicobrowser.WayBackInfo;
import nicobrowser.entity.NicoContent.Status;
import org.apache.http.HttpException;
import saccubus.worker.profile.CommentProfile;
import saccubus.worker.profile.GeneralProfile;
import saccubus.worker.profile.OutputProfile;
import saccubus.worker.profile.DownloadProfile;
import saccubus.worker.profile.ProxyProfile;

/**
 * <p>タイトル: さきゅばす</p>
 *
 * <p>説明: ニコニコ動画の動画をコメントつきで保存</p>
 *
 * <p>著作権: Copyright (c) 2007 PSI</p>
 *
 * <p>会社名: </p>
 *
 * @author 未入力
 * @version 1.0
 */
public class Download extends Worker<DownloadResult, DownloadProgress> {

    private static final Logger logger = Logger.getLogger(Download.class.getName());
    private final DownloadProfile profile;
    private final String videoId;

    public Download(DownloadProfile profile, String videoId) {
        this(profile, videoId, null);
    }

    /**
     * コンバータを構築します.
     * @param videoId 対象となる動画のID.
     * @param time
     * @param profile
     * @param listener
     * @param flag
     */
    public Download(DownloadProfile profile, String videoId, WorkerListener<DownloadProgress> listener) {
        // TODO listener登録
        super(listener);
        this.videoId = videoId;
        this.profile = profile;
    }

//    @Override
//    public Boolean call() throws Exception {
//        try {
//            final DownloadResult result = doInBackground();
//            return Boolean.valueOf(result.getResultValue());
//        } finally {
//            // TODO 何か処理が必要？
////            getStopFlag().finished();
//        }
//    }
//    // TODO Runnableを実装しなくなったので削除する
//    public void run() {
//        try {
//            call();
//        } catch (Exception ex) {
//            String text = (ex.getMessage() != null) ? ex.getMessage() : "予期しないエラー発生のため中断しました。";
//            sendText(text);
//            logger.log(Level.SEVERE, null, ex);
//        }
//    }
    @Override
    public DownloadResult call() throws Exception {

        publish(new DownloadProgress("ログイン中"));

        NicoHttpClient client = null;
        nicobrowser.VideoInfo vi = null;
        NamePattern videoNamePattern = null;
        WayBackInfo wbi = null;
        if (needsLogin()) {
            client = createClientAndLogin();
            vi = client.getVideoInfo(videoId);

            final String name = profile.getVideoProfile().getFileName();
            final String replaceFrom = profile.getGeneralProfile().getReplaceFrom();
            final String replaceTo = profile.getGeneralProfile().getReplaceTo();
            videoNamePattern = new NamePattern(name, replaceFrom, replaceTo, vi.getTitleInWatchPage());

            if (needsBackLog()) {
                final String key = client.getWayBackKey(vi);
                wbi = new WayBackInfo(key, profile.getCommentProfile().getBackLogPoint());
            }
        }

        checkStop();

        File commentFile;
        if (profile.getCommentProfile().isDownload()) {
            final CommentProfile prof = profile.getCommentProfile();
            final GeneralProfile gene = profile.getGeneralProfile();

            final NamePattern pattern = new NamePattern(prof.getFileName(), gene.getReplaceFrom(), gene.getReplaceTo(),
                    vi.getTitleInWatchPage());
            // TODO コメントファイルに{low}は使えないことをどこかに書くべきか
            final String name = pattern.createFileName(videoId, true);
            final File file = new File(profile.getCommentProfile().getDir(), name);

            commentFile = client.getCommentFile(vi, file.getPath(), wbi, profile.getCommentProfile().
                    getLengthRelatedCommentSize(),
                    profile.getCommentProfile().isDisablePerMinComment());
        } else {
            commentFile = profile.getCommentProfile().getLocalFile();
        }

        checkStop();

        File videoFile;
        GetFlvResult vf;
        if (profile.getVideoProfile().isDownload()) {
            vf = client.getFlvFile(vi, profile.getVideoProfile().getDir(), videoNamePattern,
                    Status.GET_INFO, true, new ProgressListener() {

                @Override
                public void progress(long fileSize, long downloadSize) {
                    final double vol = (double) downloadSize / (double) fileSize * 100.0;
                    publish(new DownloadProgress(String.format("ダウンロード%.2f%%", vol)));
                }
            });

            videoFile = vf.getFile();
        } else {
            videoFile = profile.getVideoProfile().getLocalFile();
        }
        return new DownloadResult(true, videoFile, commentFile);


        // TODO FFMPEG 実行開始は別タスクとして実装する.
//        if (!profile.getOutputFileSetting().isConvert()) {
//            publish(new DownloadProgress("動画・コメントを保存し、変換は行いませんでした。"));
//            return new DownloadResult(true);
//        }
//
//        if (!videoFile.isFile()) {
//            throw new IOException("入力動画ファイルが存在しません:" + videoFile.getPath());
//        }
//
//        if (profile.getOutputFileSetting().isAddComment()) {
//            if (!commentFile.isFile()) {
//                throw new IOException("入力コメントファイルが存在しません:" + commentFile.getPath());
//            }
//        } else {
//            commentFile = null;
//        }
//
//        /*ビデオ名の確定*/
//        final boolean isNotLow = (vf == null) ? true : (vf.getStatus() != Status.GET_LOW);
//        File convertedVideoFile = getOutputFileName(vi.getTitleInWatchPage(), isNotLow);
//

//        boolean res = new FfmpegCommand(getListener(), getStopFlag(), commentFile, videoFile,
//                convertedVideoFile, profile.getFfmpeg(), profile.getGeneralSetting()).execute();
//        return res;
    }

    /** @return 何かダウンロードするものがあればtrue. */
    private static boolean needsDownload(DownloadProfile profile) {
        return (profile.getVideoProfile().isDownload() || profile.getCommentProfile().isDownload());
    }

    // TODO どこかに処理を移す必要がある.
    /**
     * (ネットワーク設定以外の)設定を検証する.
     * @throws IllegalArgumentException 設定に不備がある場合.
     */
//    private void validSetting() {
//        if (profile.getOutputFileSetting().isConvert()) {
//            File a = profile.getFfmpeg().getFfmpeg();
//            if (!a.canRead()) {
//                throw new IllegalArgumentException("FFmpegが見つかりません。");
//            }
//            if (profile.getFfmpeg().getVhook().getPath().indexOf(' ') >= 0) {
//                throw new IllegalArgumentException("すいません。現在vhookライブラリには半角空白は使えません。");
//            }
//            a = profile.getFfmpeg().getVhook();
//            if (!a.canRead()) {
//                throw new IllegalArgumentException("Vhookライブラリが見つかりません。");
//            }
//            a = profile.getFfmpeg().getFont();
//            if (!a.canRead()) {
//                throw new IllegalArgumentException("フォントが見つかりません。");
//            }
//        }
//    }
    /**
     * HttpClientを生成し, ニコニコ動画サーバへログインします.
     * @return 生成したHttpClientインスタンス.
     * @throws IOException ログイン失敗.
     * @throws InterruptedException ログイン失敗.
     */
    // TODO HttpException を投げるのをやめたい. コンパイル時にHttpComponentが必要になるので.
    private NicoHttpClient createClientAndLogin() throws IOException, InterruptedException, HttpException {
        final NicoHttpClient client = createClient(profile.getProxyProfile());
        final boolean hasLogin;
        try {
            hasLogin = client.login(profile.getLoginInfo().getMail(), profile.getLoginInfo().getPassword());
            if (!hasLogin) {
                throw new IOException("login fail");
            }
        } catch (URISyntaxException ex) {
            throw new IOException("login fail", ex);
        }
        return client;
    }

    private NicoHttpClient createClient(ProxyProfile proxy) {
        if (proxy.use()) {
            return new NicoHttpClient(proxy.getHost(), proxy.getPort());
        } else {
            return new NicoHttpClient();
        }
    }

    /** @return ログインする必要があればtrue. */
    private boolean needsLogin() {
        return profile.getVideoProfile().isDownload() || profile.getCommentProfile().isDownload();
    }

    /** @return 過去ログ取得の必要があればtrue. */
    private boolean needsBackLog() {
        return profile.getCommentProfile().getBackLogPoint() >= 0L;
    }

    private void checkStop() throws InterruptedException {
        if (Thread.interrupted()) {
            throw new InterruptedException("中止要求を受け付けました");
        }
    }
}
