package yukihane.inqubus;

import java.nio.file.FileSystem;
import java.nio.file.FileSystems;
import java.nio.file.Path;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;
import javax.swing.UnsupportedLookAndFeelException;
import org.apache.commons.configuration.ConfigurationException;
import yukihane.inqubus.gui.MainFrame;

/**
 * いんきゅばす起動クラス.
 * @author yuki
 */
public class Main {

    private static final Logger logger = Logger.getLogger(Main.class.getName());

    public static void main(String args[]) {
        final FileSystem fileSystem = FileSystems.getDefault();
        final Path path = fileSystem.getPath("inqubus.xml");
        try {
            Config.INSTANCE.load(path.toString());
        } catch (ConfigurationException ex) {
            logger.log(Level.FINE, "コンフィグファイルが無いためデフォルト値で起動", ex);
        }
        setLookAndFeel();
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new MainFrame().setVisible(true);
            }
        });
    }

    private static void setLookAndFeel() {
        try {
            for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException e) {
            try {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            } catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException ex) {
                // System Look & Feel も無いことは無いだろう
            }
        }
    }
}
