package yukihane.inqubus.manager;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.FutureTask;
import saccubus.worker.Convert;
import saccubus.worker.ConvertResult;
import saccubus.worker.Download;
import saccubus.worker.DownloadResult;
import saccubus.worker.profile.ConvertProfile;
import saccubus.worker.profile.DownloadProfile;

/**
 *
 * @author yuki
 */
public class TaskManager {

    private final ExecutorService downloadExecutorService;
    private final ExecutorService convertExecutorService;
    private final Map<Integer, ManageTarget<DownloadResult>> downloadTargets = new HashMap<>();
    private final Map<Integer, ManageTarget<ConvertResult>> convertTargets = new HashMap<>();

    public TaskManager(int maxDownload, int maxConvert) {
        downloadExecutorService = Executors.newFixedThreadPool(maxDownload);
        convertExecutorService = Executors.newFixedThreadPool(maxConvert);
    }

    public synchronized boolean add(Request request) {
        final DownloadProfile dp = request.getDownloadProfile();
        final ConvertProfile cp = request.getConvertProfile();
        if (dp != null && !dp.getVideoProfile().isDownload() && !dp.getCommentProfile().isDownload()) {
            final Download task = new Download(dp, request.getVideoId(),
                    request.getDownloadProgressListener());
            final Future<DownloadResult> future = downloadExecutorService.submit(task);
            downloadTargets.put(request.getRowId(), new ManageTarget<>(request, future));
            return true;

        } else if (cp != null && cp.isConvert()) {
            final Convert task = new Convert(cp, dp.getVideoProfile().getLocalFile(), dp.getCommentProfile().
                    getLocalFile(), request.getConvertProgressListener());
            final Future<ConvertResult> future = convertExecutorService.submit(task);
            convertTargets.put(request.getRowId(), new ManageTarget<>(request, future));
            return true;
        }
        return false;
    }
}

class ManageTarget<T> {

    private final Request request;
    private final Future<T> future;

    ManageTarget(Request request, Future<T> future) {
        this.request = request;
        this.future = future;
    }

    Future<T> getFuture() {
        return future;
    }

    Request getRequest() {
        return request;
    }
}
